//======================================================================================
//
//		File: 	mchusb.cpp
//
//		Desc:	Generic Microchip USB Class for VC++
//		
//		Author:		J. Kadoya
//
//		History:	
//					Ver 1.00	2007/09/11
//					Ver 1.10	2008/09/23		CX^XΉ
//
//======================================================================================

#define		_MCHUSB_CPP

#include "stdafx.h"
#include "mchusb.h"
//#include "mpusbapi.h"
//#include "Util.h"

//
//		O[oϐ`
//
//		VendorID, ProductID ͂ŏCĂ
//
char gcVidPid[]      = "vid_04d8&pid_ff33";    // Microchip - CyberMELON TuCZX
char gcOutPipeName[] = "\\MCHP_EP1";
char gcInPipeName[]  = "\\MCHP_EP1";

CMchUSB		gMchUSB;		//  O[oCX^X


CMchUSB::CMchUSB()
{
	m_hDLL     = NULL;
	for( int i = 0; i < MAX_DEVICE; i++ )
	{
		m_hInPipe[i]  = INVALID_HANDLE_VALUE;
		m_hOutPipe[i] = INVALID_HANDLE_VALUE;
		m_bOpen[i] = FALSE;
	}
	m_nWriteTimeOut = 100;
	m_nReadTimeOut = 100;
}




CMchUSB::~CMchUSB()
{
	for( int i = 0; i < MAX_DEVICE; i++ )
	{
		if ( m_bOpen[i] ){
			Close( i );
		}
	}
}

//--------------------------------------------------------------------------------------
//
//		Function:		Initialize
//
//
//		Desc:		NX̏
//
//		Note:		ŏɃR[邱
//
//--------------------------------------------------------------------------------------
BOOL CMchUSB::Initialize()
{
	m_hDLL = ::LoadLibrary( _T("mpusbapi"));

	//	e֐̐錾
	MPUSBGetDLLVersion = (DWORD(*)(void))\
					GetProcAddress( m_hDLL,"_MPUSBGetDLLVersion");
	MPUSBGetDeviceCount = (DWORD(*)(PCHAR))\
					GetProcAddress( m_hDLL,"_MPUSBGetDeviceCount");
	MPUSBOpen = (HANDLE(*)(DWORD,PCHAR,PCHAR,DWORD,DWORD))\
					GetProcAddress( m_hDLL,"_MPUSBOpen");
	MPUSBWrite = (DWORD(*)(HANDLE,PVOID,DWORD,PDWORD,DWORD))\
					GetProcAddress( m_hDLL,"_MPUSBWrite");
	MPUSBRead = (DWORD(*)(HANDLE,PVOID,DWORD,PDWORD,DWORD))\
					GetProcAddress( m_hDLL,"_MPUSBRead");
	MPUSBReadInt = (DWORD(*)(HANDLE,PVOID,DWORD,PDWORD,DWORD))\
					GetProcAddress( m_hDLL,"_MPUSBReadInt");
	MPUSBClose = (BOOL(*)(HANDLE))GetProcAddress( m_hDLL,"_MPUSBClose");

	if(( MPUSBGetDeviceCount == NULL ) || ( MPUSBOpen == NULL ) ||
					( MPUSBWrite == NULL ) || ( MPUSBRead == NULL ) ||
					( MPUSBClose == NULL ) || ( MPUSBGetDLLVersion == NULL ) ||
					( MPUSBReadInt == NULL )){
		//	DLL Ɋ֐Ȃ
		return	FALSE;
	}

//	m_iDevCount = MPUSBGetDeviceCount( gcVidPid );	// gp
	return	TRUE;
}



//--------------------------------------------------------------------------------------
//
//		Function:		Open
//
//		In:			iDevice:	foCXԍ( 0.. )
//
//		Desc:		pCṽI[v
//
//--------------------------------------------------------------------------------------
BOOL CMchUSB::Open( int iDevice )
{

	m_bOpen[iDevice] = FALSE;
	if ( m_hDLL == NULL ){
		return	FALSE;
	}

	m_hOutPipe[iDevice] = MPUSBOpen( iDevice, gcVidPid, gcOutPipeName, MP_WRITE, 0 );
	if ( m_hOutPipe[iDevice] == INVALID_HANDLE_VALUE )
	{
		return	FALSE;
	}
	m_hInPipe[iDevice] = MPUSBOpen( iDevice, gcVidPid, gcInPipeName, MP_READ, 0 );
	if ( m_hInPipe[iDevice] == INVALID_HANDLE_VALUE )
	{
		return	FALSE;
	}
	m_bOpen[iDevice] = TRUE;
	return	TRUE;
}


//--------------------------------------------------------------------------------------
//
//		Function:		Close
//
//		In:			iDevice:	foCXԍ( 0.. )
//
//		Desc:		pCṽN[Y
//
//--------------------------------------------------------------------------------------
void CMchUSB::Close( int iDevice )
{
	if ( m_hOutPipe[iDevice] != INVALID_HANDLE_VALUE ){
		MPUSBClose( m_hOutPipe[iDevice] );
	}
	if ( m_hInPipe[iDevice] != INVALID_HANDLE_VALUE ){
		MPUSBClose( m_hInPipe[iDevice] );
	}
	m_hOutPipe[iDevice] = m_hInPipe[iDevice] = INVALID_HANDLE_VALUE;
	m_bOpen[iDevice] = FALSE;
}


//--------------------------------------------------------------------------------------
//
//		Function:		Write
//
//		Desc:		pCvւ̏
//
//		In:			iDevice:	foCXԍ( 0.. )
//					pData:		f[^ւ̃|C^
//					nLength		oCg
//
//		Ret:		݃JEg ( 0 : G[@j
//
//--------------------------------------------------------------------------------------
int CMchUSB::Write( int iDevice, BYTE *pData, int nLength )
{
	DWORD	written;

	if ( MPUSBWrite( m_hOutPipe[iDevice], pData, nLength, &written, 100 ) == FALSE )
	{
		//	ؒfꂽƔf
		Close( iDevice );
		return	0;
	}
	return	(int)written;
}


//--------------------------------------------------------------------------------------
//
//		Function:		Read
//
//		Desc:		pCv̓ǂݏo
//
//		In:			iDevice:	foCXԍ( 0.. )
//					pData:		f[^ւ̃|C^
//					nLength		f[^obt@
//
//		Ret:		ǂݍ݃JEg ( 0 : G[@j
//
//--------------------------------------------------------------------------------------
int CMchUSB::Read( int iDevice, BYTE *pData, int nLength, int nTimeOut )
{
	DWORD	read;

	if ( MPUSBRead( m_hInPipe[iDevice], pData, nLength, &read, nTimeOut ) == FALSE )
	{
		//	ؒfꂽƔf
		Close( iDevice );
		return	0;
	}
	return	(int)read;
}

